/*
===============================================================================
=== Released under GPL (GNU Public License) ==== No Any Warranty ==============
===============================================================================
*/

#include "../include/common.h"
#include "../include/main.h"

#define FONT "-misc-fixed-medium-r-normal--12-200-75-75-c-100-iso8859-1"

#ifdef _NOZVT
#warning "libzvt not supported - console will not be build."
#endif

/* Dichiara Routines esterne */
struct _OzEDIT_Object *OzEDIT_ReturnObjectFromFrameID(gint);
struct _OzEDIT_Object *OzEDIT_ReturnCurrentObject();
gchar *OzEDIT_TextEntry(gchar *,gchar *);
void OzUPDATER_Check();
void OzEDIT_ParseTextBuffer(struct _OzEDIT_Object *);
#ifdef DEBUG
gint OzEDIT_CBDebug_DumpObjectList(GtkWidget *,GdkEvent *,gpointer);
gint OzEDIT_CBDebug_DumpTopclass(GtkWidget *,GdkEvent *,gpointer);
#endif

gboolean undo_action,redo_action,critical_section;
struct _OzEDIT_TopClass topclass;

/* ==== SEGFAULT  ============================================================================================== */
void CB_SignalSEGFAULT() {
	printf("FATAL: SegFault\n");
	printf("Perhaps this is caused by a BUG in OzEDITor. Please rerun using GDB and send me an e-mail with\n");
	printf("the trace of the problem, if you can reproduce it. Ah...don't forget to read BUGS file !\n");
	exit(0);
}

/* ==== Quit  ============================================================================================== */
gint OzEDIT_CBQuit(GtkWidget *widget,GdkEvent *event,gpointer data) {
    OzEDIT_SaveConfiguration();
    gtk_main_quit();
    return FALSE;
}

/* ==== New Object ========================================================================================= */
gint OzEDIT_CBNew(GtkWidget *widget,GdkEvent *event,gpointer data) {
    OzEDIT_CreateNewObject((gchar *)NULL);
    return FALSE;
}

/* ==== Open Object ========================================================================================= */
void OzEDIT_CBOpenDialogOK(GtkWidget *widget,GtkFileSelection *fs) {
    /* Aggiungi alla Lista */
    OzEDIT_RecentAddFile((gchar *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
    OzEDIT_RecentUpdateList();
    OzEDIT_CreateNewObject((gchar *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
}

gint OzEDIT_CBOpen(GtkWidget *widget,GdkEvent *event,gpointer data) {
    GtkWidget *file_selector;
    file_selector = gtk_file_selection_new("Please select file to open.");
    g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),"clicked",G_CALLBACK(OzEDIT_CBOpenDialogOK),file_selector);
    g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),"clicked",G_CALLBACK(gtk_widget_destroy),(gpointer)file_selector);
    g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->cancel_button),"clicked",G_CALLBACK(gtk_widget_destroy),(gpointer)file_selector);
    gtk_widget_show(file_selector);        
    return FALSE;
}

/* ==== Close  ============================================================================================== */
gint OzEDIT_CBClose(GtkWidget *widget,GdkEvent *event,gpointer data) {
    struct _OzEDIT_Object *object;
    
    object = (struct _OzEDIT_Object *)OzEDIT_ReturnCurrentObject();
    if(object == NULL) return FALSE; /* Proteggiti dagli errori degli utenti */
    if(gtk_text_buffer_get_modified(object->text_buffer)==TRUE) {
	if(OzEDIT_AreYouSureDialog("Are you sure wanna close %s ?",object->name) != GTK_RESPONSE_YES) return FALSE;
    }
    OzEDIT_DestroyObject(object);
}

/* ==== Save  ============================================================================================== */
gint OzEDIT_CBSave(GtkWidget *widget,GdkEvent *event,gpointer data) {
    struct _OzEDIT_Object *object;
    object = (struct _OzEDIT_Object *)OzEDIT_ReturnCurrentObject();
#ifdef DEBUG
    g_print("DEBUG: Saving %s to %s (%d)\n",object->name,object->filename,topclass.current_frame_id);
#endif
    OzEDIT_SaveObject(object,object->filename);    
    return FALSE;
}

/* ==== SaveAs  ============================================================================================== */
void OzEDIT_CBSaveAsDialogOK(GtkWidget *widget,GtkFileSelection *fs) {
    OzEDIT_SaveObject(OzEDIT_ReturnCurrentObject(),(gchar *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
}

gint OzEDIT_CBSaveAs(GtkWidget *widget,GdkEvent *event,gpointer data) {
    GtkWidget *file_selector;
    file_selector = gtk_file_selection_new("Save as...");
    g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),"clicked",G_CALLBACK(OzEDIT_CBSaveAsDialogOK),file_selector);
    g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),"clicked",G_CALLBACK(gtk_widget_destroy),(gpointer)file_selector);
    g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->cancel_button),"clicked",G_CALLBACK(gtk_widget_destroy),(gpointer)file_selector);
    gtk_widget_show(file_selector);        
    return FALSE;
}

/* ==== Config  ============================================================================================== */
gint OzEDIT_CBConfig(GtkWidget *widget,GdkEvent *event,gpointer data) {
    OzEDIT_ConfigureDialog();
    return FALSE;
}


/* ==== Find ============================================================================================== */
gint OzEDIT_CBFind(GtkWidget *widget,GdkEvent *event,gpointer data) {
    OzEDIT_FindDialog();
    return FALSE;
}

/* ==== Set Bookmark ============================================================================================== */
gint OzEDIT_CBSetBookmark(GtkWidget *widget,GdkEvent *event,gpointer data) {
    struct _OzEDIT_Object *object;
    struct _OzEDIT_Bookmark *bookmark;
    GtkTextIter iter;

    object = (struct _OzEDIT_Object *)OzEDIT_ReturnCurrentObject();
    if(object == NULL) return TRUE;

    bookmark = g_malloc0(sizeof(struct _OzEDIT_Bookmark));

    /* Prelevare il punto del cursore nel TextBuffer */
    gtk_text_buffer_get_iter_at_mark(object->text_buffer,&iter,gtk_text_buffer_get_insert(object->text_buffer));
    /* Chiedere il nome del Bookmark (Oppure uso default name ?) */ 
    sprintf(bookmark->name,"Bookmark %02d\0",object->bk_count);
    
    g_strlcpy(bookmark->bk_name,(gchar *)OzEDIT_TextEntry("Name of BookMark ?",bookmark->name),64);
    /* Aggiungere Mark al TextBuffer */
    gtk_text_buffer_create_mark(object->text_buffer,bookmark->name,&iter,TRUE);
    /* Aggiornare lista ogetti */
    gtk_tree_store_append(GTK_TREE_STORE(topclass.treestore),&bookmark->treeiter,&object->treeiter);
    gtk_tree_store_set(GTK_TREE_STORE(topclass.treestore),&bookmark->treeiter,0,bookmark->bk_name,-1);
#ifndef _NOGDKPIXBUF
    /* Aggiungere il PIXBUF al TextBuffer */
    gtk_text_iter_forward_to_line_end(&iter);
    gtk_text_buffer_insert_pixbuf(object->text_buffer,&iter,topclass.pixbuf_bookmark);
#endif
    /* Aggiungi alla lista... */
    object->bookmarks = g_list_append(object->bookmarks,bookmark);
    object->bk_count++;
    return FALSE;
}


/* ==== Info  ============================================================================================== */
gint OzEDIT_CBInfo(GtkWidget *widget,GdkEvent *event,gpointer data) {
    OzEDIT_AboutDialog();
    return FALSE;
}

/* ==== WSSave =============================================================================================== */
void OzEDIT_CBWSSaveOK(GtkWidget *widget,GtkFileSelection *fs) {
    OzEDIT_SaveWorkspace((gchar *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
}

gint OzEDIT_CBWSSave(GtkWidget *widget,GdkEvent *event,gpointer data) {
    GtkWidget *file_selector;
    file_selector = gtk_file_selection_new("Save workspace as...");
    g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),"clicked",G_CALLBACK(OzEDIT_CBWSSaveOK),file_selector);
    g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),"clicked",G_CALLBACK(gtk_widget_destroy),(gpointer)file_selector);
    g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->cancel_button),"clicked",G_CALLBACK(gtk_widget_destroy),(gpointer)file_selector);
    gtk_widget_show(file_selector);
    return FALSE;
}

/* ==== WSClose =============================================================================================== */
gint OzEDIT_CBWSClose(GtkWidget *widget,GdkEvent *event,gpointer data) {
    OzEDIT_CloseWorkspace();
    return FALSE;
}

/* ==== EditUndo =============================================================================================== */
gint OzEDIT_CBEditUndo(GtkWidget *widget,GdkEvent *event,gpointer data) {
    struct _OzEDIT_Object *object;
    object = (struct _OzEDIT_Object *)OzEDIT_ReturnCurrentObject();
    if(object == NULL) return 0;
    OzEDIT_DoUndo((struct _OzEDIT_Object *)object);
}

/* ==== EditRedo =============================================================================================== */
gint OzEDIT_CBEditRedo(GtkWidget *widget,GdkEvent *event,gpointer data) {
    struct _OzEDIT_Object *object;
    object = (struct _OzEDIT_Object *)OzEDIT_ReturnCurrentObject();
    if(object == NULL) return 0;
    OzEDIT_DoRedo((struct _OzEDIT_Object *)object);
}

/* ==== Misc CallBacks ========================================================================================= */
void OzEDIT_CBNotebookChangePage(GtkWidget *widget,GtkNotebookPage *page,guint page_num,gpointer data) {
    struct _OzEDIT_Object *object;
    gint i;
    topclass.current_frame_id = 0;
    topclass.notebook_current_page = page_num;
    for(i=0;i<g_list_length(topclass.objects);i++) {
	object = g_list_nth_data(topclass.objects,i);    
	if(topclass.notebook_current_page == gtk_notebook_page_num(GTK_NOTEBOOK(widget),GTK_WIDGET(object->frame))) {
	    topclass.current_frame_id = object->frame_id;
	}
    }    
#ifdef DEBUG
    g_print("Switched to notebook page:%d - Frame ID:%d\n",topclass.notebook_current_page,topclass.current_frame_id);
#endif    
}

void OzEDIT_CBRowActivated(GtkTreeView *tree_view,GtkTreePath *path,GtkTreeViewColumn *column) {
}

#ifdef _THREADEDCOLORIZER
GMutex *mutex_colorizer;

void OzEDIT_CBModified(GtkTextBuffer *textbuffer,struct _OzEDIT_Object *object) {
    if(g_mutex_trylock(mutex_colorizer)) {
        object->colorized = FALSE;
#ifdef DEBUG
        g_print("Text Buffer at %s modified\n",object->name);
#endif    
	g_mutex_unlock(mutex_colorizer);
    }
}
#else
void OzEDIT_CBModified(GtkTextBuffer *textbuffer,struct _OzEDIT_Object *object) {
    object->colorized = FALSE;
    object->status = OBJECT_STATUS_UNSAVED;
#ifdef DEBUG
    g_print("Text Buffer at %s modified\n",object->name);
#endif    
    
}
#endif

#define MENU_TEXTPOPUP_ITEMS 3

static char *popup_textmenu_items[MENU_TEXTPOPUP_ITEMS][2] = {
	"Set Bookmarks Here","set_bmark",
	"Undo","undo",
	"Redo","redo"
};

static void OzEDIT_CBPopulatePopupMenuActivate(gchar *menu) {
#ifdef DEBUG
    g_print("POPUP Menu %s activated\n",menu);
#endif    
    if(g_ascii_strcasecmp(menu,"redo") == 0) { /* REDO */
	OzEDIT_DoRedo((struct _OzEDIT_Object *)OzEDIT_ReturnCurrentObject());
    }
    if(g_ascii_strcasecmp(menu,"undo") == 0) { /* UNDO */
	OzEDIT_DoUndo((struct _OzEDIT_Object *)OzEDIT_ReturnCurrentObject());
    }
    if(g_ascii_strcasecmp(menu,"set_bmark") == 0) { /* SET BOOKMARK */
	OzEDIT_CBSetBookmark((GtkWidget *)NULL,(GdkEvent *)NULL,NULL);
    }
#warning "OzEDIT_CBPopulatePopupMenuActivate UNFINSHED !"    
}

void OzEDIT_CBPopulatePopup(GtkTextView *textview,GtkMenu *menu,struct _OzEDIT_Object *object) {
    GtkWidget *mi;
    guint i;
#ifdef DEBUG
    g_print("Populate Popup for %s\n",object->name);
#endif    
    mi = gtk_menu_item_new();
    gtk_menu_shell_prepend(GTK_MENU_SHELL(menu),mi);
    gtk_widget_show(mi);
    
    for(i=0;i<MENU_TEXTPOPUP_ITEMS;i++) {
    	mi = gtk_menu_item_new_with_label((gchar *)popup_textmenu_items[i][0]);
	gtk_menu_shell_prepend(GTK_MENU_SHELL(menu),mi);
	g_signal_connect_swapped(G_OBJECT(mi), "activate", G_CALLBACK(OzEDIT_CBPopulatePopupMenuActivate),(gpointer)popup_textmenu_items[i][1]);
	gtk_widget_show(mi);
    }
}

/* ==== Popup Menu =============================================================================================== */

GtkTreeIter list_selected_iter; /* Contiene l'ITER dell'Oggetto Selezionato */

static void OzEDIT_CBPopupMenuActivate(gchar *menu) {
    GtkTreePath *treepath;
    GtkTextMark *textmark;
    GtkTextIter iter;
    gint *indices,depth,i;
    struct _OzEDIT_Object *object;
    struct _OzEDIT_Bookmark *bookmark;
    gchar *newname;
    
    treepath = gtk_tree_model_get_path(GTK_TREE_MODEL(topclass.treestore),&list_selected_iter);
    indices = gtk_tree_path_get_indices(treepath); /* Indici dell'albero */
    depth = gtk_tree_path_get_depth(treepath); /* Profondita' - 1=Oggetto 2=Bookmark */
    
#ifdef DEBUG
    g_print("POPUP Menu %s on %d:%d (%d)\n",menu,indices[0],indices[1],depth);
#endif    

    if(g_ascii_strcasecmp(menu,"prop_object") == 0) { /* Go Here (or Focus) Object */
        object = g_list_nth_data(topclass.objects,indices[0]); /* Prendo Oggetto */
	if(depth == 1) { /* Rinomina l'Oggetto */
    	    OzEDIT_ObjectProperties(object);
	} else {
	    bookmark = g_list_nth_data(object->bookmarks,indices[1]);	
    	    OzEDIT_BookmarkProperties(object,bookmark);
	}
    }
    if(g_ascii_strcasecmp(menu,"go_here") == 0) { /* Go Here (or Focus) Object */
        object = g_list_nth_data(topclass.objects,indices[0]); /* Prendo Oggetto */
        /* Vai al TAB del Notebook che lo contiene */
        gtk_notebook_set_current_page(GTK_NOTEBOOK(topclass.notebook),gtk_notebook_page_num(GTK_NOTEBOOK(topclass.notebook),object->frame));
	if(depth == 2) { /* Ora, se era selezionato un Bookmark, vai al Bookmark */
	    bookmark = g_list_nth_data(object->bookmarks,indices[1]);
	    /* Acquisisci il Bookmark*/
	    textmark = gtk_text_buffer_get_mark(object->text_buffer,bookmark->name);
	    /* Genera l'ITER */
	    gtk_text_buffer_get_iter_at_mark(object->text_buffer,&iter,textmark);
	    /* Posizionati */
	    gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(object->w_textview),&iter,0,FALSE,0,0);
	    gtk_text_buffer_place_cursor(object->text_buffer,&iter);
	    /* Focalizza */ 
	    gtk_widget_grab_focus(object->w_textview);
	}
    }
    if(g_ascii_strcasecmp(menu,"rename_object") == 0) { /* Rename Object */
        object = g_list_nth_data(topclass.objects,indices[0]); /* Prendo Oggetto */
	if(depth == 1) { /* Rinomina l'Oggetto */
	    newname = OzEDIT_TextEntry("Object's New Name ?",object->name);
	    g_strlcpy(object->name,newname,255);
	    gtk_label_set_text(GTK_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(topclass.notebook),object->frame)),object->name);
	    gtk_tree_store_set(GTK_TREE_STORE(topclass.treestore),&list_selected_iter,0,object->name,-1);
	} else { /* Rinomina il BookMark */
	    bookmark = g_list_nth_data(object->bookmarks,indices[1]);	
	    newname = OzEDIT_TextEntry("Bookmark's New Name ?",bookmark->bk_name);
	    g_strlcpy(bookmark->bk_name,newname,64);
	    gtk_tree_store_set(GTK_TREE_STORE(topclass.treestore),&list_selected_iter,0,bookmark->bk_name,-1);
	}
    }
    
    if(g_ascii_strcasecmp(menu,"clear_object") == 0) { /* Clear (or Close) Object  */
        object = g_list_nth_data(topclass.objects,indices[0]); /* Prendo Oggetto */
	if(depth == 1) { /* Chiudi Oggetto */
	    if((gtk_text_buffer_get_modified(object->text_buffer))&&(OzEDIT_AreYouSureDialog("Are you sure wanna close %s ?",object->name) != GTK_RESPONSE_YES)) return;
	    OzEDIT_DestroyObject(object);	
	} else { /* Cancella Bookmark */
	    bookmark = g_list_nth_data(object->bookmarks,indices[1]);
	    /* cancella dal buffer */
	    gtk_text_buffer_delete_mark_by_name(object->text_buffer,bookmark->name);
	    /* Togli dal Tree */
	    gtk_tree_store_remove(topclass.treestore,&bookmark->treeiter);
	    /* Rimuovi dalla lista */
	    object->bookmarks = g_list_remove(object->bookmarks,bookmark);
	    /* Ok */
	}
    }

    g_free(treepath);
}


#define MENU_POPUP_ITEMS 4

static char *popup_menu_items[MENU_POPUP_ITEMS][2] = {
	"Go Here","go_here",
	"Rename","rename_object",
	"Clear","clear_object",
	"Properties","prop_object"
};

GtkWidget *OzEDIT_CreatePopupMenu() {
	GtkWidget *menu,*menu_item;
	gint i;
	
	menu = gtk_menu_new();
	
	for(i=0;i<MENU_POPUP_ITEMS;i++) {
		menu_item = gtk_menu_item_new_with_label((gchar *)popup_menu_items[i][0]);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
		g_signal_connect_swapped(G_OBJECT(menu_item), "activate", G_CALLBACK(OzEDIT_CBPopupMenuActivate),(gpointer)popup_menu_items[i][1]);
		gtk_widget_show(menu_item);
	}
	return menu;
}

static gint OzEDIT_CBPopupMenu(GtkWidget *widget, GdkEvent *event,struct _OzEDIT_Object *object)
{
	GtkWidget *menu;
	GdkEventButton *event_button;
	GtkTreeSelection *tree_selection;

	g_return_val_if_fail(widget != NULL, FALSE);
	g_return_val_if_fail(event != NULL, FALSE);
	
	tree_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(topclass.maintree));
	if(!gtk_tree_selection_get_selected(tree_selection,NULL,&list_selected_iter)) {
	    return;
	}
	
	menu = OzEDIT_CreatePopupMenu();
	
	/* Se click con DESTRO ... */
	if(event->type == GDK_BUTTON_PRESS) {
		event_button = (GdkEventButton *)event;
		if (event_button->button == 3)
		{
	  		gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,event_button->button,event_button->time);
	 		return TRUE;
		}
	}
	/* Doppio click con SINISTRO ! */
	if (event->type == GDK_2BUTTON_PRESS)	{
		event_button = (GdkEventButton *)event;
		if (event_button->button == 1)
		{
			OzEDIT_CBPopupMenuActivate("go_here"); /* Default Action on Double-Click */
    		}
	}
	return FALSE;
}

/* === ZvtTerm ========================================================================================= */
#ifndef _NOZVT
static void OzEDIT_CBConsoleSetHints(GtkWidget *term,GtkWidget *main_window)
{
	GdkGeometry hints;

	hints.base_width = (GTK_WIDGET(topclass.console)->style->xthickness * 2) + 2;
	hints.base_height = (GTK_WIDGET(topclass.console)->style->ythickness * 2);

	hints.width_inc = ZVT_TERM(topclass.console)->charwidth;
	hints.height_inc = ZVT_TERM(topclass.console)->charheight;
	hints.min_width = hints.base_width + hints.width_inc;
	hints.min_height = hints.base_height + hints.height_inc;

	gtk_window_set_geometry_hints(GTK_WINDOW(main_window),GTK_WIDGET(topclass.console),&hints,GDK_HINT_RESIZE_INC|GDK_HINT_MIN_SIZE|GDK_HINT_BASE_SIZE);
}
#endif

/* ==== Called Periodically ====================================================================================== */
gint OzEDIT_CBIdle(gpointer data) {
    if(topclass.need_update) {
	topclass.need_update = FALSE;
	OzEDIT_InfoDialog(topclass.update_message,NULL);
    }
}

gint OzEDIT_CBTimeout(gpointer data) {
    struct _OzEDIT_Object *object;
    gchar buffer[512];
    GtkTextMark *mark;
    GtkTextIter iter;
    if(critical_section) return;
    object = (struct _OzEDIT_Object *)OzEDIT_ReturnCurrentObject();
    if(object == NULL) return TRUE;
    mark = gtk_text_buffer_get_insert(object->text_buffer);
    gtk_text_buffer_get_iter_at_mark(object->text_buffer,&iter,mark);
    sprintf(buffer,"File:%s  Lines:%d  Chars:%d  Cursor: %d:%d\0",object->filename,gtk_text_buffer_get_line_count(object->text_buffer),gtk_text_buffer_get_char_count(object->text_buffer),gtk_text_iter_get_line(&iter),gtk_text_iter_get_line_offset(&iter));	
    if(gtk_text_buffer_get_modified(object->text_buffer)) {
        strcat(buffer," MODIFIED ");
    } 
    gtk_label_set_text(GTK_LABEL(object->w_entry),buffer);
    return TRUE;
}


gint OzEDIT_CBColorize(gpointer data) {
    struct _OzEDIT_Object *object;
    GThread *thr_updater;
    GError **thr_error;
    if(critical_section) return;
    if(topclass.object_counter < 1) return TRUE;
    if(OzEDIT_Config.colorize_code) {
        object = OzEDIT_ReturnCurrentObject();
	if(object == NULL) return TRUE;
	if(object->colorized == FALSE) {
#ifdef DEBUG
	    g_print("Colorizing.\n");
#endif 
#ifdef _THREADEDCOLORIZER
#ifdef G_THREADS_ENABLED
	    if(g_thread_supported()) {
		g_mutex_lock(mutex_colorizer);
		thr_updater = g_thread_create((GThreadFunc)OzEDIT_ParseTextBuffer,object,FALSE,thr_error);
		g_mutex_unlock(mutex_colorizer);
	    } 
#else
    	    OzEDIT_ParseTextBuffer(object);
#endif
#else
    	    OzEDIT_ParseTextBuffer(object);
#endif
	    object->colorized = TRUE;
	}
    }
    return TRUE;
}


gint OzEDIT_CBUpdater(gpointer data) {
    if(critical_section) return;
#ifdef OZEDIT_PRERELEASE
    OzEDIT_InfoDialog("This is a PRERELEASE version !\n Please check for final release at http://ozeditor.zerozone.it \n",NULL);
#else
    if(OzEDIT_Config.autocheck_for_updates) OzUPDATER_Check();
#endif
    return FALSE;
}


/* ============================================================================================ */

static GtkItemFactoryEntry menu_items[] = {
  { "/_File",         	NULL,         	NULL,           	0, "<Branch>" },
  { "/File/_New",	"<control>N", 	(GtkItemFactoryCallback)OzEDIT_CBNew,		0, "<Item>" },  
  { "/File/_Open",	"<control>O", 	(GtkItemFactoryCallback)OzEDIT_CBOpen,		0, "<Item>" },  
  { "/File/_Save", 	"<control>S", 	(GtkItemFactoryCallback)OzEDIT_CBSave, 		0, "<Item>" },
  { "/File/_SaveAs", 	NULL, 		(GtkItemFactoryCallback)OzEDIT_CBSaveAs,	0, "<Item>" },
  { "/File/_Close", 	"<control>C",	(GtkItemFactoryCallback)OzEDIT_CBClose,		0, "<Item>" }, 
  { "/File/sep1",     	NULL,         	NULL,           	0, "<Separator>" },
  { "/File/Configure", 	NULL,		(GtkItemFactoryCallback)OzEDIT_CBConfig,  	0, "<Item>" },
  { "/File/sep2",     	NULL,         	NULL,           	0, "<Separator>" },
  { "/File/Quit",	"<control><alt>X", 	(GtkItemFactoryCallback)OzEDIT_CBQuit,  	0, "<Item>" },
  { "/_Edit",		NULL,		NULL,			0, "<Branch>" },
  { "/Edit/_Undo", 	"<control>Z", 		(GtkItemFactoryCallback)OzEDIT_CBEditUndo, 	0, "<Item>" },  
  { "/Edit/_Redo", 	"<control>R", 		(GtkItemFactoryCallback)OzEDIT_CBEditRedo, 	0, "<Item>" },    
  { "/_Workspace",	NULL,		NULL,			0, "<Branch>" },
  { "/Workspace/_Save WorkSpace", NULL, 	(GtkItemFactoryCallback)OzEDIT_CBWSSave, 	0, "<Item>" },  
  { "/Workspace/_Close WorkSpace", NULL, 	(GtkItemFactoryCallback)OzEDIT_CBWSClose, 	0, "<Item>" },  
#ifdef DEBUG
  { "/_Debug",	NULL,		NULL,			0, "<Branch>" },
  { "/Debug/_Dump Object List", NULL,	(GtkItemFactoryCallback)OzEDIT_CBDebug_DumpObjectList, 	0, "<Item>" },  
  { "/Debug/_Dump Topclass", 	NULL, 	(GtkItemFactoryCallback)OzEDIT_CBDebug_DumpTopclass, 	0, "<Item>" },  
#endif  
  { "/_Help",         	NULL,         	NULL,           	0, "<LastBranch>" },
  { "/_Help/Check Release",   	NULL,         	(GtkItemFactoryCallback)OzEDIT_CBUpdater,	0, "<Item>" },  
  { "/_Help/About",   	NULL,         	(GtkItemFactoryCallback)OzEDIT_CBInfo,		0, "<Item>" },
};

static gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

extern char      **environ;		
static char      **env;
static char      **env_copy;
static int         winid_pos;

int main(int argc, char *argv[])
{
	GtkWidget *w_main_window,*w_main_table,*w_main_vbox,*w_main_menubar,*w_main_toolbar,*w_main_hpaned,*w_main_vpaned,*w_scrollable,*w_frameconsole;
	GtkWidget *w_icon,*w_button_open,*w_button_new,*w_button_close,*w_console_hbox,*w_console_scrollbar,*w_left_notebook;	
	GtkWidget *w_label_objects,*w_label_filebrowser,*w_label_recentfile;
	GdkPixmap *v_pixmap;
	GdkBitmap *v_mask;
	GtkStyle *v_style;
	GtkItemFactory *v_item_factory;
	GtkAccelGroup *v_accel_group;
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;	
	GString *shell, *name;
	int i, c, login_shell;
	char **p, *fontname=FONT;
	struct passwd *pw;

	signal(SIGSEGV,CB_SignalSEGFAULT);

	gtk_init(&argc,&argv);
	
	/* Crea la Finestra Principale */
	w_main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(w_main_window),"OzEDITor");
	gtk_widget_set_size_request(GTK_WIDGET(w_main_window), 640, 480);
	g_signal_connect(G_OBJECT(w_main_window),"destroy",G_CALLBACK(OzEDIT_CBQuit),NULL);
	gtk_container_set_border_width(GTK_CONTAINER(w_main_window), 2);
	
	gtk_widget_realize(w_main_window);

	/* Carica i vari PIXBUFs */
#ifndef _NOGDKPIXBUF
	topclass.pixbuf_bookmark = gdk_pixbuf_new_from_xpm_data((gchar **)bookmark_xpm);
#endif
	/* Table Container */
  	w_main_table = gtk_table_new(12,3,FALSE);
	gtk_container_add(GTK_CONTAINER(w_main_window), w_main_table);
	gtk_widget_show(w_main_table);

	w_main_vbox = gtk_vbox_new(FALSE,1);
	gtk_container_set_border_width(GTK_CONTAINER(w_main_vbox), 1);
	gtk_table_attach_defaults(GTK_TABLE(w_main_table),w_main_vbox,0,2,0,1);

	/* Menu */
	v_accel_group = gtk_accel_group_new();
	v_item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR,"<main>",v_accel_group);
	gtk_item_factory_create_items(v_item_factory,nmenu_items,menu_items,NULL);
	gtk_window_add_accel_group (GTK_WINDOW(w_main_window),v_accel_group);
	w_main_menubar = gtk_item_factory_get_widget(v_item_factory, "<main>");
	gtk_widget_show(w_main_menubar);

	/* ToolBar */
	w_main_toolbar = gtk_toolbar_new();
	gtk_widget_set_size_request(GTK_WIDGET(w_main_toolbar), 640, 40);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(w_main_toolbar),GTK_ORIENTATION_HORIZONTAL);
	gtk_toolbar_set_style(GTK_TOOLBAR(w_main_toolbar),GTK_TOOLBAR_ICONS);
	gtk_container_set_border_width(GTK_CONTAINER(w_main_toolbar),1);

	/* Toolbar Icons */
	gtk_toolbar_set_style(GTK_TOOLBAR(w_main_toolbar), GTK_TOOLBAR_ICONS);
	gtk_toolbar_insert_stock(GTK_TOOLBAR(w_main_toolbar),GTK_STOCK_NEW,"New",NULL,(GtkItemFactoryCallback)OzEDIT_CBNew,NULL,-1);
	gtk_toolbar_insert_stock(GTK_TOOLBAR(w_main_toolbar),GTK_STOCK_OPEN,"Open",NULL,(GtkItemFactoryCallback)OzEDIT_CBOpen,NULL,-1);
	gtk_toolbar_insert_stock(GTK_TOOLBAR(w_main_toolbar),GTK_STOCK_SAVE,"Save",NULL,(GtkItemFactoryCallback)OzEDIT_CBSave,NULL,-1);
	gtk_toolbar_insert_stock(GTK_TOOLBAR(w_main_toolbar),GTK_STOCK_SAVE_AS,"SaveAs",NULL,(GtkItemFactoryCallback)OzEDIT_CBSaveAs,NULL,-1);
	gtk_toolbar_insert_stock(GTK_TOOLBAR(w_main_toolbar),GTK_STOCK_CLOSE,"Close",NULL,(GtkItemFactoryCallback)OzEDIT_CBClose,NULL,-1);
	gtk_toolbar_append_space(GTK_TOOLBAR(w_main_toolbar));
	gtk_toolbar_insert_stock(GTK_TOOLBAR(w_main_toolbar),GTK_STOCK_FIND,"Find",NULL,(GtkItemFactoryCallback)OzEDIT_CBFind,NULL,-1);
	gtk_toolbar_insert_stock(GTK_TOOLBAR(w_main_toolbar),GTK_STOCK_INDEX,"Set Bookmark",NULL,(GtkItemFactoryCallback)OzEDIT_CBSetBookmark,NULL,-1);
	gtk_toolbar_append_space(GTK_TOOLBAR(w_main_toolbar));
	gtk_toolbar_insert_stock(GTK_TOOLBAR(w_main_toolbar),GTK_STOCK_PREFERENCES,"Preferences",NULL,(GtkItemFactoryCallback)OzEDIT_CBConfig,NULL,-1);

	gtk_toolbar_append_space(GTK_TOOLBAR(w_main_toolbar));
	
	gtk_widget_show(w_main_toolbar);

	/* Pack it all together */
	gtk_box_pack_start(GTK_BOX(w_main_vbox),w_main_menubar, FALSE, TRUE, 0);
	gtk_box_pack_end(GTK_BOX(w_main_vbox),w_main_toolbar, TRUE, TRUE, 0);

	gtk_widget_show(w_main_vbox);

	/* Main Console Widget - Per la CONSOLE */
	
	w_frameconsole = gtk_frame_new("Console");
	gtk_widget_set_size_request(w_frameconsole,640,80);
	gtk_widget_show(w_frameconsole);
	
#ifndef _NOZVT
#warning "Compile with ZVT Lib support"
	w_console_hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_set_spacing(GTK_BOX(w_console_hbox), 2);
        gtk_container_set_border_width(GTK_CONTAINER(w_console_hbox), 2);

	topclass.console = zvt_term_new_with_size(80,24);
	zvt_term_set_font_name(ZVT_TERM(topclass.console), fontname); 
	zvt_term_set_blink(ZVT_TERM(topclass.console), TRUE);
	zvt_term_set_bell(ZVT_TERM(topclass.console), TRUE);
	zvt_term_set_scrollback(ZVT_TERM(topclass.console),50);
	zvt_term_set_scroll_on_keystroke(ZVT_TERM (topclass.console), TRUE);
	zvt_term_set_scroll_on_output(ZVT_TERM(topclass.console), FALSE);
	zvt_term_set_background(ZVT_TERM(topclass.console), NULL, 0, 0);
	zvt_term_set_wordclass(ZVT_TERM(topclass.console), "-A-Za-z0-9/_:.,?+%=");
	
	/* Focus on ZvtWidget - Tnx to Ken Steen */
	g_signal_connect(G_OBJECT(topclass.console),"button_press_event",G_CALLBACK(gtk_widget_grab_focus),topclass.console); 
        g_signal_connect_after(G_OBJECT(topclass.console),"realize",G_CALLBACK(OzEDIT_CBConsoleSetHints),w_main_window);
	gtk_widget_show(topclass.console);

	w_console_scrollbar = gtk_vscrollbar_new(GTK_ADJUSTMENT(ZVT_TERM(topclass.console)->adjustment));
	GTK_WIDGET_UNSET_FLAGS(w_console_scrollbar, GTK_CAN_FOCUS);
	gtk_widget_show(w_console_scrollbar);
	
	gtk_box_pack_start(GTK_BOX(w_console_hbox),w_console_scrollbar, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(w_console_hbox),topclass.console, 1, 1, 0);
	gtk_widget_show(w_console_hbox);

	gtk_container_add(GTK_CONTAINER(w_frameconsole), w_console_hbox);

	/* Genera la copia dell'Ambiente di Esecuzione */
	env = environ;
	for (p = env; *p; p++);
	i = p - env;
	env_copy = (char **) g_malloc (sizeof (char *) * (i + 3));
	for (i = 0, p = env; *p; p++) {
    	    if (strncmp (*p, "TERM=", 5) == 0)
		env_copy [i++] = "TERM=xterm";
    	    else if ((strncmp (*p, "COLUMNS=", 8) == 0) || (strncmp (*p, "LINES=", 6) == 0))
		continue;
    	    else
		env_copy [i++] = *p;
	}

	env_copy[i++] = "COLORTERM=zterm";
	winid_pos = i++;
	env_copy[winid_pos] = "TEST";
	env_copy[i] = NULL;

	/* Adesso Esegui la BASH */
	switch (zvt_term_forkpty(ZVT_TERM(topclass.console), ZVT_TERM_DO_UTMP_LOG | ZVT_TERM_DO_WTMP_LOG | ZVT_TERM_DO_LASTLOG)) {
	    case -1:
		perror("ERROR: unable to fork:");
		exit(1);
		break;
	    case 0:
	        /* get shell from passwd */
    	        pw = getpwuid(getuid());
    	        if (pw) {
	    	    shell = g_string_new(pw->pw_shell);
		    if (login_shell) {
		        name = g_string_new("-");
		    } else {
		        name = g_string_new("");
		    }
		    g_string_append(name, strrchr(pw->pw_shell, '/'));
    		} else {
		    shell = g_string_new("/bin/sh");
		    if (login_shell) {
		        name = g_string_new("-sh");
		    } else {
		        name = g_string_new("sh");
		    }
		}
    		execle (shell->str, name->str, NULL, env_copy);
    		perror ("Could not exec\n");
    		_exit (127);
	    default:
		break;
	 }
#endif
	/* Scrollable per la LISTA TREE */
	w_scrollable =  gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(w_scrollable),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_widget_set_size_request(GTK_WIDGET(w_scrollable), -1, 380);
	gtk_container_set_border_width(GTK_CONTAINER(w_scrollable),1);
	gtk_widget_show(w_scrollable);

	/* Tree List Widget */
	topclass.treestore = gtk_tree_store_new(2,G_TYPE_STRING,G_TYPE_STRING);
	topclass.maintree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(topclass.treestore));
	g_object_unref(G_OBJECT(topclass.treestore));
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(topclass.maintree), TRUE);
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(topclass.maintree)),GTK_SELECTION_BROWSE);
	
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Objects",renderer,"text", 0,NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(topclass.maintree),GTK_TREE_VIEW_COLUMN(column));
	
	g_signal_connect(GTK_TREE_VIEW(topclass.maintree),"row-activated",G_CALLBACK(OzEDIT_CBRowActivated),GTK_TREE_MODEL(topclass.treestore));
        g_signal_connect(G_OBJECT(topclass.maintree),"button_press_event",G_CALLBACK(OzEDIT_CBPopupMenu),NULL);
	
	gtk_widget_set_size_request(topclass.maintree,200,0);
	gtk_container_add(GTK_CONTAINER(w_scrollable),topclass.maintree);

	gtk_widget_show(topclass.maintree);

	/* NoteBook - Appartiene alla Classe Principale */
	topclass.notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(topclass.notebook), GTK_POS_TOP);
	g_signal_connect(GTK_OBJECT(topclass.notebook),"switch-page",G_CALLBACK(OzEDIT_CBNotebookChangePage),NULL);
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(topclass.notebook),TRUE);
	gtk_widget_show(topclass.notebook);

	w_label_objects = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(w_label_objects),"<small>Objects</small>");

	w_label_filebrowser = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(w_label_filebrowser),"<small>FileBrowser</small>");

	w_label_recentfile = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(w_label_recentfile),"<small>RecentFiles</small>");

	/* Left Notebook - Object List / File List */
	w_left_notebook = gtk_notebook_new();
	gtk_notebook_append_page(GTK_NOTEBOOK(w_left_notebook),w_scrollable,w_label_objects);
	gtk_notebook_append_page(GTK_NOTEBOOK(w_left_notebook),(GtkWidget *)OzEDIT_FileBrowser(),w_label_filebrowser);
	gtk_notebook_append_page(GTK_NOTEBOOK(w_left_notebook),(GtkWidget *)OzEDIT_RecentFiles(),w_label_recentfile);

	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w_left_notebook),GTK_POS_LEFT);
	gtk_widget_show(w_left_notebook);

	w_main_hpaned = gtk_hpaned_new();
	gtk_paned_add1(GTK_PANED(w_main_hpaned),w_left_notebook);
	gtk_paned_add2(GTK_PANED(w_main_hpaned),topclass.notebook);
	gtk_widget_show(w_main_hpaned);    

	w_main_vpaned = gtk_vpaned_new();
	gtk_table_attach_defaults(GTK_TABLE(w_main_table),w_main_vpaned, 0, 3, 1, 11);
	gtk_paned_add1(GTK_PANED(w_main_vpaned),w_main_hpaned);
	gtk_paned_add2(GTK_PANED(w_main_vpaned),w_frameconsole);
	gtk_widget_show(w_main_vpaned);    

	/* Status Bar */
	topclass.statusbar = gtk_statusbar_new();      
	gtk_table_attach_defaults(GTK_TABLE(w_main_table),topclass.statusbar,0,2,11,12);
	gtk_widget_set_size_request(GTK_WIDGET(topclass.statusbar), 640, 1);
	gtk_widget_show(topclass.statusbar);
	
	gtk_widget_show(w_main_window); 

	topclass.object_counter = 0;
    
	gtk_timeout_add(200,OzEDIT_CBTimeout,NULL);
	gtk_timeout_add(5000,OzEDIT_CBUpdater,NULL);
	gtk_timeout_add(500,OzEDIT_CBColorize,NULL);
	gtk_idle_add(OzEDIT_CBIdle,NULL);	

	OzEDIT_LoadConfiguration();
	OzEDIT_RecentUpdateList();

#ifdef G_THREADS_ENABLED
	if(!g_thread_supported()) {
    	    g_thread_init(NULL);    
#ifdef _THREADEDCOLORIZER
#warning "You choose THREADED COLORIZER - This is an instable feature ! You are warned."
	    mutex_colorizer = g_mutex_new();
#endif
	}
#endif

	critical_section = FALSE;
	topclass.need_update = FALSE;
	
	gtk_main();
}
