/*
===============================================================================
=== Released under GPL (GNU Public License) ==== No Any Warranty ==============
===============================================================================
*/

#include "../include/common.h"
#include "../include/objects.h"

extern gboolean critical_section;
void OzEDIT_CBModified(GtkTextBuffer *,struct _OzEDIT_Object *);
void OzEDIT_CBPopulatePopup(GtkTextView *,GtkMenu *,struct _OzEDIT_Object *);
void OzEDIT_UndoRedoInit(struct _OzEDIT_Object *);

/* ==== Objects Functions =========================================================================================== */

void OzEDIT_DestroyObject(struct _OzEDIT_Object *object) {
#ifdef DEBUG
    g_print("DEBUG: DestroyObject: %s \n",object->name);
#endif
    critical_section = TRUE;
    /* Rimuovi dalla Tree View degli OGGETTI */
    gtk_tree_store_remove(GTK_TREE_STORE(topclass.treestore),&object->treeiter);
    /* Rimuovi dal NOTEBOOK */
    gtk_notebook_remove_page(GTK_NOTEBOOK(topclass.notebook),gtk_notebook_page_num(GTK_NOTEBOOK(topclass.notebook),object->frame));
    /* This cause a SEGFAULT....why ? */
/* REMOVED - gtk_widget_destroy(object->frame); - CAUSE A LOT OF SEGFAULTs */
    /* Libera la memoria */
    g_list_free(object->bookmarks);
    g_list_free(object->listundo);
    g_list_free(object->listredo);    
    /* Rimuovi dalla lista degli OGGETTI */
    topclass.objects = g_list_remove(topclass.objects,object);
    /* Finally free the object... */
    g_free(object);
    critical_section = FALSE;
}

struct _OzEDIT_Object *OzEDIT_CreateNewObject(gchar *filename) {
    struct _OzEDIT_Object *new_object;
    PangoFontDescription *font;

    if(g_str_has_suffix(filename,".ows")) { /* E' un WorkSpace */
#ifdef DEBUG
	g_print("CreateNewObject: OWS File\n");
#endif
	OzEDIT_OpenWorkspace(filename);
	return NULL;
    }

    /* Alloca memoria x il nuovo oggetto */
    new_object = g_malloc0(sizeof(struct _OzEDIT_Object));
    /* Crea il FRAME di Contenimento */
    new_object->frame = gtk_frame_new("");
    /* Crea VBOX */
    new_object->w_vbox = gtk_vbox_new(FALSE,1);
    gtk_container_add(GTK_CONTAINER(new_object->frame),new_object->w_vbox);

    /* Crea Text Entry */
    new_object->w_entry = gtk_label_new("");
    gtk_widget_show(new_object->w_entry);
    
    /* Crea SCROLLER WINDOW */
    new_object->w_scrolled = gtk_scrolled_window_new(NULL,NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(new_object->w_scrolled),GTK_POLICY_ALWAYS,GTK_POLICY_ALWAYS);
        
    gtk_container_set_border_width(GTK_CONTAINER(new_object->w_scrolled),1);
    gtk_widget_show(new_object->w_scrolled);
    /* Crea Text Widget */
    new_object->w_textview = gtk_text_view_new();
    new_object->text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(new_object->w_textview));
    
    /* Setta il FONT */
    gtk_widget_modify_font(new_object->w_textview,OzEDIT_Config.font_textbuffer);
    /* Crea i TAG x il Text Buffer */
    new_object->tag_blue = gtk_text_buffer_create_tag(new_object->text_buffer,"blue-foreground","foreground","blue",NULL);
    new_object->tag_red = gtk_text_buffer_create_tag(new_object->text_buffer,"red-foreground","foreground","red",NULL);
    new_object->tag_green = gtk_text_buffer_create_tag(new_object->text_buffer,"green-foreground","foreground","green",NULL);
    new_object->tag_gray = gtk_text_buffer_create_tag(new_object->text_buffer,"gray-foreground","foreground","gray",NULL);
    new_object->tag_graybg = gtk_text_buffer_create_tag(new_object->text_buffer,"gray-background","background","gray",NULL);
    /* Vai x il Text View */    
    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(new_object->w_textview),GTK_WRAP_NONE);
    gtk_text_view_set_editable(GTK_TEXT_VIEW(new_object->w_textview),TRUE);
    gtk_container_add(GTK_CONTAINER(new_object->w_scrolled),new_object->w_textview);
    gtk_widget_show(new_object->w_textview);
    /* ID - Univoco - Del Frame */    
    new_object->frame_id = topclass.object_counter++;
    /* Adesso CARICA il FILE */
    if(filename != NULL) {
	/* Apri il file */
	OzEDIT_LoadObject(new_object,filename);		
	/* Setta le proprieta' del FRAME */
	strncpy(new_object->filename,filename,255);
	sprintf(new_object->name,"%s (%d)",filename,new_object->frame_id);
        new_object->status = OBJECT_STATUS_OK;
    } else {
	/* Setta le proprieta' del FRAME */
	sprintf(new_object->name,"Untitled (%d)",new_object->frame_id);
        new_object->status = OBJECT_STATUS_NEW;
	gtk_text_buffer_set_modified(new_object->text_buffer,FALSE); 
    }
    /* Adesso attaccati al CallBack per il colorizer */
    g_signal_connect(G_OBJECT(new_object->text_buffer),"modified-changed",G_CALLBACK(OzEDIT_CBModified),new_object);
    g_signal_connect(G_OBJECT(new_object->text_buffer),"begin-user-action",G_CALLBACK(OzEDIT_CBModified),new_object);
    /* Attacca il segnale populate-popup */
    g_signal_connect(G_OBJECT(new_object->w_textview),"populate-popup",G_CALLBACK(OzEDIT_CBPopulatePopup),new_object);
    
    new_object->w_label = gtk_label_new(new_object->name);
    gtk_frame_set_label(GTK_FRAME(new_object->frame),new_object->name);
    gtk_widget_show(new_object->frame);
    gtk_notebook_append_page(GTK_NOTEBOOK(topclass.notebook), new_object->frame, new_object->w_label);

    /* Impacchetta... */
    gtk_box_pack_start(GTK_BOX(new_object->w_vbox),new_object->w_entry,FALSE,TRUE,0);
    gtk_box_pack_end(GTK_BOX(new_object->w_vbox),new_object->w_scrolled,TRUE,TRUE,0);
    gtk_widget_show(new_object->w_vbox);

    /* Compila tabella PROPRIETA' OGGETTO */
    new_object->status = OBJECT_STATUS_NEW;
    /* Aggiungi alla LISTA degli OGGETTI */
    topclass.objects = g_list_append(topclass.objects,new_object);
    /* Adesso aggiungi al TREE VIEW */
    gtk_tree_store_append(GTK_TREE_STORE(topclass.treestore),&new_object->treeiter,NULL);
    gtk_tree_store_set(GTK_TREE_STORE(topclass.treestore),&new_object->treeiter,0,new_object->name,-1);
    /* Inzializza Undo & Redo */
    OzEDIT_UndoRedoInit(new_object);
    /* Finito */
    new_object->colorized = FALSE;
    gtk_text_buffer_set_modified(new_object->text_buffer,FALSE);
    return (struct _OzEDIT_Object *)new_object;
}


guint OzEDIT_CheckUnsavedObject() {
    gint i;
    struct _OzEDIT_Object *object;
    for(i=0;i<g_list_length(topclass.objects);i++) {
	object = g_list_nth_data(topclass.objects,i);
	if(gtk_text_buffer_get_modified(object->text_buffer)) return 1;
    }
    return 0;
}

struct _OzEDIT_Object *OzEDIT_ReturnObjectFromFrameID(guint id) {
    gint i;
    struct _OzEDIT_Object *object;
    for(i=0;i<g_list_length(topclass.objects);i++) {
	object = g_list_nth_data(topclass.objects,i);
	if(object->frame_id == id) return (struct _OzEDIT_Object *)object;
    }
    return (struct _OzEDIT_Object *)NULL;
}

struct _OzEDIT_Object *OzEDIT_ReturnCurrentObject() {
    guint page;
    page = gtk_notebook_get_current_page(GTK_NOTEBOOK(topclass.notebook));    
    gint i;
    struct _OzEDIT_Object *object;
    for(i=0;i<g_list_length(topclass.objects);i++) {
	object = g_list_nth_data(topclass.objects,i);
	if(gtk_notebook_page_num(GTK_NOTEBOOK(topclass.notebook),object->frame) == page) return (struct _OzEDIT_Object *)object;
    }
    return (struct _OzEDIT_Object *)NULL;
}
